from argparse import ArgumentParser

from loguru import logger

from autotrain.infer.text_generation import TextGenerationInference

from ..trainers.clm import train as train_llm
from ..trainers.utils import LLMTrainingParams
from . import BaseAutoTrainCommand


def run_llm_command_factory(args):
    return RunAutoTrainLLMCommand(
        args.train,
        args.deploy,
        args.inference,
        args.data_path,
        args.train_split,
        args.valid_split,
        args.text_column,
        args.model,
        args.learning_rate,
        args.num_train_epochs,
        args.train_batch_size,
        args.eval_batch_size,
        args.warmup_ratio,
        args.gradient_accumulation_steps,
        args.optimizer,
        args.scheduler,
        args.weight_decay,
        args.max_grad_norm,
        args.seed,
        args.add_eos_token,
        args.block_size,
        args.use_peft,
        args.lora_r,
        args.lora_alpha,
        args.lora_dropout,
        args.training_type,
        args.train_on_inputs,
        args.logging_steps,
        args.project_name,
        args.evaluation_strategy,
        args.save_total_limit,
        args.save_strategy,
        args.auto_find_batch_size,
        args.fp16,
        args.push_to_hub,
        args.use_int8,
        args.model_max_length,
        args.repo_id,
        args.use_int4,
        args.trainer,
        args.target_modules,
    )


class RunAutoTrainLLMCommand(BaseAutoTrainCommand):
    @staticmethod
    def register_subcommand(parser: ArgumentParser):
        run_llm_parser = parser.add_parser(
            "llm",
            description="✨ Run AutoTrain LLM training/inference/deployment",
        )
        run_llm_parser.add_argument(
            "--train",
            help="Train the model",
            required=False,
            action="store_true",
        )
        run_llm_parser.add_argument(
            "--deploy",
            help="Deploy the model",
            required=False,
            action="store_true",
        )
        run_llm_parser.add_argument(
            "--inference",
            help="Run inference",
            required=False,
            action="store_true",
        )
        run_llm_parser.add_argument(
            "--data_path",
            help="Train dataset to use",
            required=False,
            type=str,
        )
        run_llm_parser.add_argument(
            "--train_split",
            help="Test dataset split to use",
            required=False,
            type=str,
            default="train",
        )
        run_llm_parser.add_argument(
            "--valid_split",
            help="Validation dataset split to use",
            required=False,
            type=str,
            default=None,
        )
        run_llm_parser.add_argument(
            "--text_column",
            help="Text column to use",
            required=False,
            type=str,
            default="text",
        )
        run_llm_parser.add_argument(
            "--model",
            help="Model to use",
            required=False,
            type=str,
        )
        run_llm_parser.add_argument(
            "--learning_rate",
            help="Learning rate to use",
            required=False,
            type=float,
            default=3e-5,
        )
        run_llm_parser.add_argument(
            "--num_train_epochs",
            help="Number of training epochs to use",
            required=False,
            type=int,
            default=1,
        )
        run_llm_parser.add_argument(
            "--train_batch_size",
            help="Training batch size to use",
            required=False,
            type=int,
            default=2,
        )
        run_llm_parser.add_argument(
            "--eval_batch_size",
            help="Evaluation batch size to use",
            required=False,
            type=int,
            default=4,
        )
        run_llm_parser.add_argument(
            "--warmup_ratio",
            help="Warmup proportion to use",
            required=False,
            type=float,
            default=0.1,
        )
        run_llm_parser.add_argument(
            "--gradient_accumulation_steps",
            help="Gradient accumulation steps to use",
            required=False,
            type=int,
            default=1,
        )
        run_llm_parser.add_argument(
            "--optimizer",
            help="Optimizer to use",
            required=False,
            type=str,
            default="adamw_torch",
        )
        run_llm_parser.add_argument(
            "--scheduler",
            help="Scheduler to use",
            required=False,
            type=str,
            default="linear",
        )
        run_llm_parser.add_argument(
            "--weight_decay",
            help="Weight decay to use",
            required=False,
            type=float,
            default=0.0,
        )
        run_llm_parser.add_argument(
            "--max_grad_norm",
            help="Max gradient norm to use",
            required=False,
            type=float,
            default=1.0,
        )
        run_llm_parser.add_argument(
            "--seed",
            help="Seed to use",
            required=False,
            type=int,
            default=42,
        )
        run_llm_parser.add_argument(
            "--add_eos_token",
            help="Add EOS token to use",
            required=False,
            action="store_true",
        )
        run_llm_parser.add_argument(
            "--block_size",
            help="Block size to use",
            required=False,
            type=int,
            default=-1,
        )
        run_llm_parser.add_argument(
            "--use_peft",
            help="Use PEFT to use",
            required=False,
            action="store_true",
        )
        run_llm_parser.add_argument(
            "--lora_r",
            help="Lora r to use",
            required=False,
            type=int,
            default=16,
        )
        run_llm_parser.add_argument(
            "--lora_alpha",
            help="Lora alpha to use",
            required=False,
            type=int,
            default=32,
        )
        run_llm_parser.add_argument(
            "--lora_dropout",
            help="Lora dropout to use",
            required=False,
            type=float,
            default=0.05,
        )
        run_llm_parser.add_argument(
            "--training_type",
            help="Training type to use",
            required=False,
            type=str,
            default="generic",
        )
        run_llm_parser.add_argument(
            "--train_on_inputs",
            help="Train on inputs to use",
            required=False,
            action="store_true",
        )
        run_llm_parser.add_argument(
            "--logging_steps",
            help="Logging steps to use",
            required=False,
            type=int,
            default=-1,
        )
        run_llm_parser.add_argument(
            "--project_name",
            help="Output directory",
            required=False,
            type=str,
        )
        run_llm_parser.add_argument(
            "--evaluation_strategy",
            help="Evaluation strategy to use",
            required=False,
            type=str,
            default="epoch",
        )
        run_llm_parser.add_argument(
            "--save_total_limit",
            help="Save total limit to use",
            required=False,
            type=int,
            default=1,
        )
        run_llm_parser.add_argument(
            "--save_strategy",
            help="Save strategy to use",
            required=False,
            type=str,
            default="epoch",
        )
        run_llm_parser.add_argument(
            "--auto_find_batch_size",
            help="Auto find batch size True/False",
            required=False,
            action="store_true",
        )
        run_llm_parser.add_argument(
            "--fp16",
            help="FP16 True/False",
            required=False,
            action="store_true",
        )
        run_llm_parser.add_argument(
            "--push_to_hub",
            help="Push to hub True/False",
            required=False,
            action="store_true",
        )
        run_llm_parser.add_argument(
            "--use_int8",
            help="Use int8 True/False",
            required=False,
            action="store_true",
        )
        run_llm_parser.add_argument(
            "--model_max_length",
            help="Model max length to use",
            required=False,
            type=int,
            default=1024,
        )
        run_llm_parser.add_argument(
            "--repo_id",
            help="Repo id for hugging face hub",
            required=False,
            type=str,
        )
        run_llm_parser.add_argument(
            "--use_int4",
            help="Use int4 True/False",
            required=False,
            action="store_true",
        )
        run_llm_parser.add_argument(
            "--trainer",
            help="Trainer type to use",
            required=False,
            type=str,
            default="default",
        )
        run_llm_parser.add_argument(
            "--target_modules",
            help="Target modules to use",
            required=False,
            type=str,
            default=None,
        )

        run_llm_parser.set_defaults(func=run_llm_command_factory)

    def __init__(
        self,
        train,
        deploy,
        inference,
        data_path,
        train_split,
        valid_split,
        text_column,
        model,
        learning_rate,
        num_train_epochs,
        train_batch_size,
        eval_batch_size,
        warmup_ratio,
        gradient_accumulation_steps,
        optimizer,
        scheduler,
        weight_decay,
        max_grad_norm,
        seed,
        add_eos_token,
        block_size,
        use_peft,
        lora_r,
        lora_alpha,
        lora_dropout,
        training_type,
        train_on_inputs,
        logging_steps,
        project_name,
        evaluation_strategy,
        save_total_limit,
        save_strategy,
        auto_find_batch_size,
        fp16,
        push_to_hub,
        use_int8,
        model_max_length,
        repo_id,
        use_int4,
        trainer,
        target_modules,
    ):
        self.train = train
        self.deploy = deploy
        self.inference = inference
        self.data_path = data_path
        self.train_split = train_split
        self.valid_split = valid_split
        self.text_column = text_column
        self.model = model
        self.learning_rate = learning_rate
        self.num_train_epochs = num_train_epochs
        self.train_batch_size = train_batch_size
        self.eval_batch_size = eval_batch_size
        self.warmup_ratio = warmup_ratio
        self.gradient_accumulation_steps = gradient_accumulation_steps
        self.optimizer = optimizer
        self.scheduler = scheduler
        self.weight_decay = weight_decay
        self.max_grad_norm = max_grad_norm
        self.seed = seed
        self.add_eos_token = add_eos_token
        self.block_size = block_size
        self.use_peft = use_peft
        self.lora_r = lora_r
        self.lora_alpha = lora_alpha
        self.lora_dropout = lora_dropout
        self.training_type = training_type
        self.train_on_inputs = train_on_inputs
        self.logging_steps = logging_steps
        self.project_name = project_name
        self.evaluation_strategy = evaluation_strategy
        self.save_total_limit = save_total_limit
        self.save_strategy = save_strategy
        self.auto_find_batch_size = auto_find_batch_size
        self.fp16 = fp16
        self.push_to_hub = push_to_hub
        self.use_int8 = use_int8
        self.model_max_length = model_max_length
        self.repo_id = repo_id
        self.use_int4 = use_int4
        self.trainer = trainer
        self.target_modules = target_modules

        if self.train:
            if self.project_name is None:
                raise ValueError("Project name must be specified")
            if self.data_path is None:
                raise ValueError("Data path must be specified")
            if self.model is None:
                raise ValueError("Model must be specified")
            if self.push_to_hub:
                if self.repo_id is None:
                    raise ValueError("Repo id must be specified for push to hub")

        if self.inference:
            tgi = TextGenerationInference(self.project_name, use_int4=self.use_int4, use_int8=self.use_int8)
            while True:
                prompt = input("User: ")
                if prompt == "exit()":
                    break
                print(f"Bot: {tgi.chat(prompt)}")

    def run(self):
        logger.info("Running LLM")
        logger.info(f"Train: {self.train}")
        if self.train:
            params = LLMTrainingParams(
                model_name=self.model,
                data_path=self.data_path,
                train_split=self.train_split,
                valid_split=self.valid_split,
                text_column=self.text_column,
                learning_rate=self.learning_rate,
                num_train_epochs=self.num_train_epochs,
                train_batch_size=self.train_batch_size,
                eval_batch_size=self.eval_batch_size,
                warmup_ratio=self.warmup_ratio,
                gradient_accumulation_steps=self.gradient_accumulation_steps,
                optimizer=self.optimizer,
                scheduler=self.scheduler,
                weight_decay=self.weight_decay,
                max_grad_norm=self.max_grad_norm,
                seed=self.seed,
                add_eos_token=self.add_eos_token,
                block_size=self.block_size,
                use_peft=self.use_peft,
                lora_r=self.lora_r,
                lora_alpha=self.lora_alpha,
                lora_dropout=self.lora_dropout,
                training_type=self.training_type,
                train_on_inputs=self.train_on_inputs,
                logging_steps=self.logging_steps,
                project_name=self.project_name,
                evaluation_strategy=self.evaluation_strategy,
                save_total_limit=self.save_total_limit,
                save_strategy=self.save_strategy,
                auto_find_batch_size=self.auto_find_batch_size,
                fp16=self.fp16,
                push_to_hub=self.push_to_hub,
                use_int8=self.use_int8,
                model_max_length=self.model_max_length,
                repo_id=self.repo_id,
                use_int4=self.use_int4,
                trainer=self.trainer,
                target_modules=self.target_modules,
            )
            train_llm(params)



# coding=utf-8
# Copyright 2020-2021 The HuggingFace AutoTrain Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
# pylint: enable=line-too-long
import os


# ignore bnb warnings
os.environ["BITSANDBYTES_NOWELCOME"] = "1"
# os.environ["HF_HUB_DISABLE_PROGRESS_BARS"] = "1"
__version__ = "0.6.7"



from itertools import chain

import torch
from datasets import Dataset
from loguru import logger
from peft import PeftModel
from pydantic import BaseModel, Field
from transformers import AutoModelForCausalLM, AutoTokenizer


IGNORE_INDEX = -100
DEFAULT_PAD_TOKEN = "[PAD]"
DEFAULT_EOS_TOKEN = "</s>"
DEFAULT_BOS_TOKEN = "</s>"
DEFAULT_UNK_TOKEN = "</s>"
TARGET_MODULES = {
    "Salesforce/codegen25-7b-multi": "q_proj,k_proj,v_proj,o_proj,down_proj,up_proj,gate_proj",
}

MODEL_CARD = """
---
tags:
- autotrain
- text-generation
widget:
- text: "I love AutoTrain because "
---

# Model Trained Using AutoTrain
"""


class LLMTrainingParams(BaseModel):
    model_name: str = Field("gpt2", title="Model name")
    data_path: str = Field("data", title="Data path")
    train_split: str = Field("train", title="Train data config")
    valid_split: str = Field(None, title="Validation data config")
    text_column: str = Field("text", title="Text column")
    huggingface_token: str = Field(None, title="Huggingface token")
    learning_rate: float = Field(3e-5, title="Learning rate")
    num_train_epochs: int = Field(1, title="Number of training epochs")
    train_batch_size: int = Field(2, title="Training batch size")
    eval_batch_size: int = Field(4, title="Evaluation batch size")
    warmup_ratio: float = Field(0.1, title="Warmup proportion")
    gradient_accumulation_steps: int = Field(1, title="Gradient accumulation steps")
    optimizer: str = Field("adamw_torch", title="Optimizer")
    scheduler: str = Field("linear", title="Scheduler")
    weight_decay: float = Field(0.0, title="Weight decay")
    max_grad_norm: float = Field(1.0, title="Max gradient norm")
    seed: int = Field(42, title="Seed")
    add_eos_token: bool = Field(True, title="Add EOS token")
    block_size: int = Field(-1, title="Block size")
    use_peft: bool = Field(False, title="Use PEFT")
    lora_r: int = Field(16, title="Lora r")
    lora_alpha: int = Field(32, title="Lora alpha")
    lora_dropout: float = Field(0.05, title="Lora dropout")
    training_type: str = Field("generic", title="Training type")
    train_on_inputs: bool = Field(False, title="Train on inputs")
    logging_steps: int = Field(-1, title="Logging steps")
    project_name: str = Field("Project Name", title="Output directory")
    evaluation_strategy: str = Field("epoch", title="Evaluation strategy")
    save_total_limit: int = Field(1, title="Save total limit")
    save_strategy: str = Field("epoch", title="Save strategy")
    auto_find_batch_size: bool = Field(False, title="Auto find batch size")
    fp16: bool = Field(False, title="FP16")
    push_to_hub: bool = Field(False, title="Push to hub")
    use_int8: bool = Field(False, title="Use int8")
    model_max_length: int = Field(1024, title="Model max length")
    repo_id: str = Field(None, title="Repo id")
    use_int4: bool = Field(False, title="Use int4")
    trainer: str = Field("default", title="Trainer type")
    target_modules: str = Field(None, title="Target modules")


def get_target_modules(config):
    if config.target_modules is None:
        return TARGET_MODULES.get(config.model_name)
    return config.target_modules.split(",")


def process_data(data, tokenizer, config):
    data = data.to_pandas()
    data = data.fillna("")

    data = data[[config.text_column]]
    if config.add_eos_token:
        data[config.text_column] = data[config.text_column] + tokenizer.eos_token
    data = Dataset.from_pandas(data)
    return data


def group_texts(examples, config):
    # Concatenate all texts.
    concatenated_examples = {k: list(chain(*examples[k])) for k in examples.keys()}
    total_length = len(concatenated_examples[list(examples.keys())[0]])
    # We drop the small remainder, we could add padding if the model supported it instead of this drop, you can
    # customize this part to your needs.
    if total_length >= config.block_size:
        total_length = (total_length // config.block_size) * config.block_size
    else:
        total_length = 0
    # Split by chunks of max_len.
    result = {
        k: [t[i : i + config.block_size] for i in range(0, total_length, config.block_size)]
        for k, t in concatenated_examples.items()
    }
    result["labels"] = result["input_ids"].copy()
    return result


def tokenize(examples, tokenizer, config):
    output = tokenizer(examples[config.text_column])
    return output


def _tokenize(prompt, tokenizer, config):
    result = tokenizer(
        prompt,
        truncation=True,
        max_length=tokenizer.model_max_length,
        padding=False,
        return_tensors=None,
    )
    if result["input_ids"][-1] != tokenizer.eos_token_id and config.add_eos_token:
        if len(result["input_ids"]) >= tokenizer.model_max_length:
            result["input_ids"] = result["input_ids"][:-1]
            result["attention_mask"] = result["attention_mask"][:-1]
        result["input_ids"].append(tokenizer.eos_token_id)
        result["attention_mask"].append(1)

    result["labels"] = result["input_ids"].copy()

    return result


def merge_adapter(base_model_path, target_model_path, adapter_path):
    logger.info("Loading adapter...")
    model = AutoModelForCausalLM.from_pretrained(
        base_model_path,
        torch_dtype=torch.float16,
        low_cpu_mem_usage=True,
        trust_remote_code=True,
    )

    model = PeftModel.from_pretrained(model, adapter_path)

    tokenizer = AutoTokenizer.from_pretrained(
        base_model_path,
        trust_remote_code=True,
    )
    model = model.merge_and_unload()

    logger.info("Saving target model...")
    model.save_pretrained(target_model_path)
    tokenizer.save_pretrained(target_model_path)


def create_model_card():
    return MODEL_CARD.strip()



from dataclasses import dataclass
from typing import Optional

import pandas as pd
from datasets import Dataset
from sklearn.model_selection import train_test_split


RESERVED_COLUMNS = ["autotrain_text", "autotrain_label"]
LLM_RESERVED_COLUMNS = ["autotrain_prompt", "autotrain_context", "autotrain_response", "autotrain_prompt_start"]


@dataclass
class TextBinaryClassificationPreprocessor:
    train_data: pd.DataFrame
    text_column: str
    label_column: str
    username: str
    project_name: str
    token: str
    valid_data: Optional[pd.DataFrame] = None
    test_size: Optional[float] = 0.2
    seed: Optional[int] = 42

    def __post_init__(self):
        # check if text_column and label_column are in train_data
        if self.text_column not in self.train_data.columns:
            raise ValueError(f"{self.text_column} not in train data")
        if self.label_column not in self.train_data.columns:
            raise ValueError(f"{self.label_column} not in train data")
        # check if text_column and label_column are in valid_data
        if self.valid_data is not None:
            if self.text_column not in self.valid_data.columns:
                raise ValueError(f"{self.text_column} not in valid data")
            if self.label_column not in self.valid_data.columns:
                raise ValueError(f"{self.label_column} not in valid data")

        # make sure no reserved columns are in train_data or valid_data
        for column in RESERVED_COLUMNS:
            if column in self.train_data.columns:
                raise ValueError(f"{column} is a reserved column name")
            if self.valid_data is not None:
                if column in self.valid_data.columns:
                    raise ValueError(f"{column} is a reserved column name")

    def split(self):
        if self.valid_data is not None:
            return self.train_data, self.valid_data
        else:
            train_df, valid_df = train_test_split(
                self.train_data,
                test_size=self.test_size,
                random_state=self.seed,
                stratify=self.train_data[self.label_column],
            )
            train_df = train_df.reset_index(drop=True)
            valid_df = valid_df.reset_index(drop=True)
            return train_df, valid_df

    def prepare_columns(self, train_df, valid_df):
        train_df.loc[:, "autotrain_text"] = train_df[self.text_column]
        train_df.loc[:, "autotrain_label"] = train_df[self.label_column]
        valid_df.loc[:, "autotrain_text"] = valid_df[self.text_column]
        valid_df.loc[:, "autotrain_label"] = valid_df[self.label_column]

        # drop text_column and label_column
        train_df = train_df.drop(columns=[self.text_column, self.label_column])
        valid_df = valid_df.drop(columns=[self.text_column, self.label_column])
        return train_df, valid_df

    def prepare(self):
        train_df, valid_df = self.split()
        train_df, valid_df = self.prepare_columns(train_df, valid_df)
        train_df = Dataset.from_pandas(train_df)
        valid_df = Dataset.from_pandas(valid_df)
        train_df.push_to_hub(
            f"{self.username}/autotrain-data-{self.project_name}",
            split="train",
            private=True,
            token=self.token,
        )
        valid_df.push_to_hub(
            f"{self.username}/autotrain-data-{self.project_name}",
            split="validation",
            private=True,
            token=self.token,
        )
        return train_df, valid_df


class TextMultiClassClassificationPreprocessor(TextBinaryClassificationPreprocessor):
    pass


class TextSingleColumnRegressionPreprocessor(TextBinaryClassificationPreprocessor):
    def split(self):
        if self.valid_data is not None:
            return self.train_data, self.valid_data
        else:
            train_df, valid_df = train_test_split(
                self.train_data,
                test_size=self.test_size,
                random_state=self.seed,
            )
            train_df = train_df.reset_index(drop=True)
            valid_df = valid_df.reset_index(drop=True)
            return train_df, valid_df


@dataclass
class LLMPreprocessor:
    train_data: pd.DataFrame
    username: str
    project_name: str
    token: str
    valid_data: Optional[pd.DataFrame] = None
    test_size: Optional[float] = 0.2
    seed: Optional[int] = 42
    context_column: Optional[str] = None
    prompt_start_column: Optional[str] = None
    text_column: Optional[str] = None
    prompt_column: Optional[str] = None
    response_column: Optional[str] = None

    def __post_init__(self):
        # user can either provide text_column or prompt_column and response_column
        if self.text_column is not None and (self.prompt_column is not None or self.response_column is not None):
            raise ValueError("Please provide either text_column or prompt_column and response_column")

        if self.text_column is not None:
            # if text_column is provided, use it for prompt_column and response_column
            self.prompt_column = self.text_column
            self.response_column = self.text_column

        # check if text_column and response_column are in train_data
        if self.prompt_column not in self.train_data.columns:
            raise ValueError(f"{self.prompt_column} not in train data")
        if self.response_column not in self.train_data.columns:
            raise ValueError(f"{self.response_column} not in train data")
        # check if text_column and response_column are in valid_data
        if self.valid_data is not None:
            if self.prompt_column not in self.valid_data.columns:
                raise ValueError(f"{self.prompt_column} not in valid data")
            if self.response_column not in self.valid_data.columns:
                raise ValueError(f"{self.response_column} not in valid data")

        # make sure no reserved columns are in train_data or valid_data
        for column in RESERVED_COLUMNS + LLM_RESERVED_COLUMNS:
            if column in self.train_data.columns:
                raise ValueError(f"{column} is a reserved column name")
            if self.valid_data is not None:
                if column in self.valid_data.columns:
                    raise ValueError(f"{column} is a reserved column name")

    def split(self):
        if self.valid_data is not None:
            return self.train_data, self.valid_data
        else:
            train_df, valid_df = train_test_split(
                self.train_data,
                test_size=self.test_size,
                random_state=self.seed,
            )
            train_df = train_df.reset_index(drop=True)
            valid_df = valid_df.reset_index(drop=True)
            return train_df, valid_df

    def prepare_columns(self, train_df, valid_df):
        if self.text_column is not None:
            train_df.loc[:, "autotrain_text"] = train_df[self.text_column]
            valid_df.loc[:, "autotrain_text"] = valid_df[self.text_column]

            # drop text_column and label_column
            train_df = train_df.drop(columns=[self.text_column])
            valid_df = valid_df.drop(columns=[self.text_column])
            return train_df, valid_df
        else:
            train_df.loc[:, "autotrain_prompt"] = train_df[self.prompt_column]
            valid_df.loc[:, "autotrain_prompt"] = valid_df[self.prompt_column]

            train_df.loc[:, "autotrain_response"] = train_df[self.response_column]
            valid_df.loc[:, "autotrain_response"] = valid_df[self.response_column]

            train_df = train_df.drop(columns=[self.prompt_column, self.response_column])
            valid_df = valid_df.drop(columns=[self.prompt_column, self.response_column])

            if self.context_column is not None:
                train_df.loc[:, "autotrain_context"] = train_df[self.context_column]
                valid_df.loc[:, "autotrain_context"] = valid_df[self.context_column]

                train_df = train_df.drop(columns=[self.context_column])
                valid_df = valid_df.drop(columns=[self.context_column])

            if self.prompt_start_column is not None:
                train_df.loc[:, "autotrain_prompt_start"] = train_df[self.prompt_start_column]
                valid_df.loc[:, "autotrain_prompt_start"] = valid_df[self.prompt_start_column]

                train_df = train_df.drop(columns=[self.prompt_start_column])
                valid_df = valid_df.drop(columns=[self.prompt_start_column])

            return train_df, valid_df

    def prepare(self):
        train_df, valid_df = self.split()
        train_df, valid_df = self.prepare_columns(train_df, valid_df)
        train_df = Dataset.from_pandas(train_df)
        valid_df = Dataset.from_pandas(valid_df)
        train_df.push_to_hub(
            f"{self.username}/autotrain-data-{self.project_name}",
            split="train",
            private=True,
            token=self.token,
        )
        valid_df.push_to_hub(
            f"{self.username}/autotrain-data-{self.project_name}",
            split="validation",
            private=True,
            token=self.token,
        )
        return train_df, valid_df
